#define FLUSH while(getchar() != '\n') // For Main.C

// For Recognition.c
#define MAXSENSORS 16
#define CALIBRATION_READS 500000
#define X 0
#define Y 1
#define Z 2
#define AS_DEGREES 57.2957795
#define MIN 0.1
#define MAX 4		// checking if it is moving or not
#define HOWLONG 60		// 'how long a gesture is'

#define MAXLOC 0
#define MINLOC 1

#define MXSHORT 35	// max >_>
#define MNSHORT 15	// min >_>
#define MXLONG 55	// max >_>
#define MNLONG 37	// min >_>

#define MAXKERNEL 23

#define SHORTMOTION 25
#define SHORTKERNEL 5
#define SHORTNORM 30
#define LONGMOTION 47
#define LONGKERNEL 7
#define LONGNORM 60

#define IMPOSSIBLE 666.666

// For Data.c
#define MAX2 24		// 'how long a gesture is'
#define NUMFILES 27

#pragma comment(lib, "winmm.lib")	// needed for some reason to use joystick stuff

// For Recognition.c
void recognition(void);
unsigned int joyGetNumDevicesConnected(void);
int getRawSensorData(int sensorID, double *AccX, double *AccY, double *AccZ, unsigned int *Buttons);
int getCalibratedSensorData(int sensorID, double *AccX, double *AccY, double *AccZ, unsigned int *Buttons, double CalibData[MAXSENSORS][3]);
void calibrateAllSensors(double calibXYZ[MAXSENSORS][3]);

void reorderArray(double* incoming, double* outgoing, int index, int howFarBack);

int openFile(double* arrX, double* arrY, double* arrZ, char* filename);
double cartesianDifference(double* ref, double* norm, int endOfArray);
int checkGesture(double* accX, double* accY, double* accZ, double** refs, struct GESTURESTRUCT* gesture, double threshold, int numReadings);
int shortMotion(double* accX, double* accY, double* accZ, double* stretchedX, double* stretchedY, double* stretchedZ, struct GESTURESTRUCT* gestureStruct, int numReadings);
int longMotion(double* accX, double* accY, double* accZ, double* stretchedX, double* stretchedY, double* stretchedZ, struct GESTURESTRUCT* gestureStruct, int numReadings);


enum GESTURE { DOWN, UP, LEFT, RIGHT, ALPHA, CAST, HADOUK } gest;

struct GESTURESTRUCT {
	enum GESTURE gesture;
	int numXCrits;
	int numYCrits;
	int numZCrits;
	int critLocations[3][10];	// 3 axis, max 10 crits
	int maxOrMin[3];			// 0 for max, 1 for min
};

// For Data.c
int openDataBank(struct GESTURESTRUCT* gestureStruct, enum GESTURE gest);
int generateTextFileShort();
int generateTextFileLong();

int superFunctionShort(double** data, int* dataSize, struct GESTURESTRUCT* gestureStruct, int axis, int toNorm, double** norm, int* restart);
int superFunctionLong(double** data, int* dataSize, struct GESTURESTRUCT* gestureStruct, int axis, int toNorm, double** norm, int* restart);

int getCorrectSample(int folder, int fileNumber, int maxReadings, int gesture);
int calculateifMaxOrMin(double* arr, int i);

void newSmoothFunction(double* arr, double* smooth, int endOfArray, int* smoothSize, int kernel);

int findNumCritPoints(double* arr, int arraySize);
int findAbsMaxMinLocation(double* arr, int* locations, int locationsSize, int maxOrMin);
int* newFindCriticalPoints(double* smoothArr, int size, int arraySize, int numCrits, int maxOrMin);

int openAllFiles(double** xData, double** yData, double** zData, int gesture);
int openAllFilesSized(double** xData, double** yData, double** zData, int* dataSize, int gesture);

int normaliseArray(double* arr, double* norm, struct GESTURESTRUCT* gestureStruct, int axis, int from, int to);
int simpleNormOne(double* arr, double* norm, double from, double to);

int bresenhamForOne(double* arr, double* norm, double from, double to, int fromBeginIndex, int fromEndIndex, int toBeginIndex, int toEndIndex);

int normaliseOne(double* arr, double* norm, int* critLocations, struct GESTURESTRUCT* gestureStruct, int axis, int from, int to);
int normaliseOneCrit(double* arr, double* norm, int* critLocations, struct GESTURESTRUCT* gestureStruct, int axis, int from, int to);

void stretchArrays(double** normsX, double** normsY, double** normsZ, double** stretchedX, double** stretchedY, double** stretchedZ, int numToStretched, int normlength, struct GESTURESTRUCT* gestureStruct);
int interpolateData(double* norm, int endOfArray, int j);
int averageArrays(double** arrsX, double** arrsY, double** arrsZ, double* avgX, double* avgY, double* avgZ, int normLength);
